﻿using System.Text;

namespace ILoan.Tools.Infrastructure.Services;

public class RuleFileGeneratorService(RulesContext context, IConfiguration configuration)
{
    private readonly int _startRuleId = configuration.GetValue<int>("Parameters:StartRuleId");
    private int _startSeq = configuration.GetValue<int>("Parameters:StartSeqNo");
    private readonly string _root = configuration.GetValue<string>("Parameters:GeneratorRoot");
    private readonly string _workitem = configuration.GetValue<string>("Parameters:Workitem");

    public async Task GenerateRuleFilesAsync()
    {
        if (!_root.EndsWith("migrations"))
        {
            if (Directory.Exists(_root)) Directory.Delete(_root, true);
            if (!Directory.Exists(_root)) Directory.CreateDirectory(_root);
        }

        var rules = await context.CoreRules
          .Include(r => r.CoreRuleCriteria)
          .Include(r => r.CoreRuleCriteriaOperators)
          .Where(rule => rule.ID >= _startRuleId)
          .ToListAsync();

        foreach (var rule in rules.Where(r => _workitem.Contains(r.WorkItemId.ToString())).OrderBy(x => x.ID))
        {
            var fileName = $"V2.{_startSeq:000}__{rule.WorkItemId}_Rules_{rule.ID}_{rule.Title}.sql";

            if (rule.Update > 0)
            {
                fileName = $"V2.{_startSeq:000}__{rule.WorkItemId}_Rules_{rule.ID}_{rule.Title}_Update({rule.Update}).sql";
            }

            var sql = GenerateSqlStatement(rule);
            Console.WriteLine($"Generating file: {fileName}");
            await File.WriteAllTextAsync(Path.Combine(_root, fileName), sql, Encoding.UTF8);
            _startSeq++;
        }
    }

    private string GenerateSqlStatement(CoreRule rule)
    {
        var sqlBuilder = new StringBuilder();

        if (rule.Update == 0)
        {
            // SQL for inserting CoreRule
            sqlBuilder.AppendLine($@"
INSERT INTO core.""Core_Rules"" (
    ""ID"", ""RuleName"", ""Source"", ""DefaultThreshold"", ""Blocked"", ""TriggersManualReview"", 
    ""SuggestedResolution"", ""TriggerReason"", ""UseLastRecord"", ""ApplicantType"", ""FilterOnClickthrough"",
    ""RunBeforeStatus"", ""DisplaySection"", ""WarningField"", ""DutchRuleName"", ""DutchTriggerReason"", 
    ""DutchSuggestedResolution"", ""DutchFilterOnClickthrough"", ""RuleExplanation"", ""DutchRuleExplanation"", ""Scope""
) OVERRIDING SYSTEM VALUE VALUES (
    {rule.ID}, '{EscapeSingleQuote(rule.RuleName)}', '{EscapeSingleQuote(rule.Source)}', {rule.DefaultThreshold?.ToString() ?? "NULL"}, 
    {rule.Blocked.ToString().ToUpper()}, {rule.TriggersManualReview.ToString().ToUpper()}, '{EscapeSingleQuote(rule.SuggestedResolution)}', 
    '{EscapeSingleQuote(rule.TriggerReason)}', {rule.UseLastRecord?.ToString().ToUpper() ?? "NULL"}, '{EscapeSingleQuote(rule.ApplicantType)}', 
    '{EscapeSingleQuote(rule.FilterOnClickthrough)}', '{EscapeSingleQuote(rule.RunBeforeStatus)}', '{EscapeSingleQuote(rule.DisplaySection)}', 
    '{EscapeSingleQuote(rule.WarningField)}', '{EscapeSingleQuote(rule.DutchRuleName)}', '{EscapeSingleQuote(rule.DutchTriggerReason)}', 
    '{EscapeSingleQuote(rule.DutchSuggestedResolution)}', '{EscapeSingleQuote(rule.DutchFilterOnClickthrough)}', 
    '{EscapeSingleQuote(rule.RuleExplanation)}', '{EscapeSingleQuote(rule.DutchRuleExplanation)}', '{EscapeSingleQuote(rule.Scope)}'
);
");

            // SQL for inserting CoreRuleCriteria
            foreach (var criterion in rule.CoreRuleCriteria)
            {
                sqlBuilder.AppendLine($@"
INSERT INTO core.""Core_RuleCriteria"" (
    ""ID"", ""RuleID"", ""Property"", ""Comparison"", ""Value"", ""ValueIsProperty"", ""ReturnCount"", ""IsThresholdCriterium""
) OVERRIDING SYSTEM VALUE VALUES (
    {criterion.ID}, {criterion.RuleID}, '{EscapeSingleQuote(criterion.Property)}', '{EscapeSingleQuote(criterion.Comparison)}', '{EscapeSingleQuote(criterion.Value)}', 
    {criterion.ValueIsProperty?.ToString().ToUpper() ?? "NULL"}, {criterion.ReturnCount?.ToString().ToUpper() ?? "NULL"}, {criterion.IsThresholdCriterium?.ToString().ToUpper() ?? "NULL"}
);
");
            }

            // SQL for inserting CoreRuleCriteriaOperator
            foreach (var coreRuleCriteriaOperator in rule.CoreRuleCriteriaOperators)
            {
                sqlBuilder.AppendLine($@"
INSERT INTO core.""Core_RuleCriteriaOperator"" (
    ""ID"", ""RuleID"", ""Operator""
) OVERRIDING SYSTEM VALUE VALUES (
    {coreRuleCriteriaOperator.ID}, {coreRuleCriteriaOperator.RuleID}, '{EscapeSingleQuote(coreRuleCriteriaOperator.Operator1)}'
);
");
            }
        }
        else
        {
            // SQL for updating CoreRule
            sqlBuilder.AppendLine($@"
UPDATE core.""Core_Rules""
SET 
    ""RuleName"" = '{EscapeSingleQuote(rule.RuleName)}',
    ""Source"" = '{EscapeSingleQuote(rule.Source)}',
    ""DefaultThreshold"" = {rule.DefaultThreshold?.ToString() ?? "NULL"},
    ""Blocked"" = {rule.Blocked.ToString().ToUpper()},
    ""TriggersManualReview"" = {rule.TriggersManualReview.ToString().ToUpper()},
    ""SuggestedResolution"" = '{EscapeSingleQuote(rule.SuggestedResolution)}',
    ""TriggerReason"" = '{EscapeSingleQuote(rule.TriggerReason)}',
    ""UseLastRecord"" = {rule.UseLastRecord?.ToString().ToUpper() ?? "NULL"},
    ""ApplicantType"" = '{EscapeSingleQuote(rule.ApplicantType)}',
    ""FilterOnClickthrough"" = '{EscapeSingleQuote(rule.FilterOnClickthrough)}',
    ""RunBeforeStatus"" = '{EscapeSingleQuote(rule.RunBeforeStatus)}',
    ""DisplaySection"" = '{EscapeSingleQuote(rule.DisplaySection)}',
    ""WarningField"" = '{EscapeSingleQuote(rule.WarningField)}',
    ""DutchRuleName"" = '{EscapeSingleQuote(rule.DutchRuleName)}',
    ""DutchTriggerReason"" = '{EscapeSingleQuote(rule.DutchTriggerReason)}',
    ""DutchSuggestedResolution"" = '{EscapeSingleQuote(rule.DutchSuggestedResolution)}',
    ""DutchFilterOnClickthrough"" = '{EscapeSingleQuote(rule.DutchFilterOnClickthrough)}',
    ""RuleExplanation"" = '{EscapeSingleQuote(rule.RuleExplanation)}',
    ""DutchRuleExplanation"" = '{EscapeSingleQuote(rule.DutchRuleExplanation)}',
    ""Scope"" = '{EscapeSingleQuote(rule.Scope)}'
WHERE ""ID"" = {rule.ID};
");

            // SQL for updating CoreRuleCriteria
            foreach (var criterion in rule.CoreRuleCriteria)
            {
                sqlBuilder.AppendLine($@"
UPDATE core.""Core_RuleCriteria""
SET 
    ""Property"" = '{EscapeSingleQuote(criterion.Property)}',
    ""Comparison"" = '{EscapeSingleQuote(criterion.Comparison)}',
    ""Value"" = '{EscapeSingleQuote(criterion.Value)}',
    ""ValueIsProperty"" = {criterion.ValueIsProperty?.ToString().ToUpper() ?? "NULL"},
    ""ReturnCount"" = {criterion.ReturnCount?.ToString().ToUpper() ?? "NULL"},
    ""IsThresholdCriterium"" = {criterion.IsThresholdCriterium?.ToString().ToUpper() ?? "NULL"}
WHERE ""ID"" = {criterion.ID};
");
            }

            // SQL for updating CoreRuleCriteriaOperator
            foreach (var coreRuleCriteriaOperator in rule.CoreRuleCriteriaOperators)
            {
                sqlBuilder.AppendLine($@"
UPDATE core.""Core_RuleCriteriaOperator""
SET 
    ""Operator"" = '{EscapeSingleQuote(coreRuleCriteriaOperator.Operator1)}'
WHERE ""ID"" = {coreRuleCriteriaOperator.ID};
");
            }
        }

        return sqlBuilder.ToString();
    }

    private static string EscapeSingleQuote(string value)
    {
        return value?.Replace("'", "''");
    }
}

// Usage example:
// var context = new RulesContext();
// var generator = new RuleFileGeneratorService(context, configuration);
// await generator.GenerateRuleFilesAsync();
